@@ -8,14 +8,23 @@
 pub struct libdeflate_compressor { _unused : [ u8 ; 0 ] , }
 #[repr(C)]
 pub struct libdeflate_decompressor { _unused : [ u8 ; 0 ] , }
+
+#[repr(C)]
+pub struct libdeflate_options {
+    pub sizeof_options: usize,
+    pub malloc_func: Option<unsafe extern "C" fn(arg1: usize) -> *mut ::std::os::raw::c_void>,
+    pub free_func: Option<unsafe extern "C" fn(arg1: *mut ::std::os::raw::c_void)>,
+}
+
 pub const libdeflate_result_LIBDEFLATE_SUCCESS: libdeflate_result = 0;
 pub const libdeflate_result_LIBDEFLATE_BAD_DATA: libdeflate_result = 1;
 pub const libdeflate_result_LIBDEFLATE_SHORT_OUTPUT: libdeflate_result = 2;
 pub const libdeflate_result_LIBDEFLATE_INSUFFICIENT_SPACE: libdeflate_result = 3;
-pub type libdeflate_result = u32 ;
+pub type libdeflate_result = ::std::os::raw::c_uint;
 
 extern "C" {
     pub fn libdeflate_alloc_decompressor() -> *mut libdeflate_decompressor;
+    pub fn libdeflate_alloc_decompressor_ex(options: *const libdeflate_options) -> *mut libdeflate_decompressor;
     pub fn libdeflate_free_decompressor(decompressor: *mut libdeflate_decompressor);
 
     pub fn libdeflate_gzip_decompress(decompressor: *mut libdeflate_decompressor,
@@ -25,6 +34,14 @@ extern "C" {
                                       out_nbytes_avail: usize,
                                       actual_out_nbytes_ret: *mut usize) -> libdeflate_result;
 
+    pub fn libdeflate_gzip_decompress_ex(decompressor: *mut libdeflate_decompressor,
+                                         in_: *const ::std::os::raw::c_void,
+                                         in_nbytes: usize,
+                                         out: *mut ::std::os::raw::c_void,
+                                         out_nbytes_avail: usize,
+                                         actual_in_nbytes_ret: *mut usize,
+                                         actual_out_nbytes_ret: *mut usize) -> libdeflate_result;
+
     pub fn libdeflate_zlib_decompress(decompressor: *mut libdeflate_decompressor,
                                       in_: *const ::std::os::raw::c_void,
                                       in_nbytes: usize,
@@ -32,14 +49,33 @@ extern "C" {
                                       out_nbytes_avail: usize,
                                       actual_out_nbytes_ret: *mut usize) -> libdeflate_result;
 
+    pub fn libdeflate_zlib_decompress_ex(decompressor: *mut libdeflate_decompressor,
+                                         in_: *const ::std::os::raw::c_void,
+                                         in_nbytes: usize,
+                                         out: *mut ::std::os::raw::c_void,
+                                         out_nbytes_avail: usize,
+                                         actual_in_nbytes_ret: *mut usize,
+                                         actual_out_nbytes_ret: *mut usize) -> libdeflate_result;
+
     pub fn libdeflate_deflate_decompress(decompressor: *mut libdeflate_decompressor,
                                          in_: *const ::std::os::raw::c_void,
                                          in_nbytes: usize,
                                          out: *mut ::std::os::raw::c_void,
                                          out_nbytes_avail: usize,
                                          actual_out_nbytes_ret: *mut usize) -> libdeflate_result;
+    
+    pub fn libdeflate_deflate_decompress_ex(decompressor: *mut libdeflate_decompressor, 
+                                            in_: *const ::std::os::raw::c_void,
+                                            in_nbytes: usize,
+                                            out: *mut ::std::os::raw::c_void,
+                                            out_nbytes_avail: usize,
+                                            actual_in_nbytes_ret: *mut usize,
+                                            actual_out_nbytes_ret: *mut usize) -> libdeflate_result;
 
     pub fn libdeflate_alloc_compressor(compression_level: ::std::os::raw::c_int) -> *mut libdeflate_compressor;
+    
+    pub fn libdeflate_alloc_compressor_ex(compression_level: ::std::ffi::c_int,
+                                          options: *const libdeflate_options) -> *mut libdeflate_compressor;
 
     pub fn libdeflate_deflate_compress_bound(compressor: *mut libdeflate_compressor,
                                              in_nbytes: usize) -> usize;
@@ -106,6 +142,22 @@ mod tests {
         }
     }
 
+    #[test]
+    fn can_make_decompressor_with_null_malloc_free() {
+        unsafe {
+            let options = libdeflate_options {
+                sizeof_options: std::mem::size_of::<libdeflate_options>(),
+                malloc_func: None,
+                free_func: None,
+            };
+            let ptr = libdeflate_alloc_decompressor_ex(&options);
+
+            assert!(!ptr.is_null());
+
+            libdeflate_free_decompressor(ptr);
+        }
+    }
+
     #[test]
     fn making_compressor_with_negative_compression_lvl_fails() {
         unsafe {
